@@ -42,13 +42,17 @@ use trillium_testing::prelude::*;
 
 assert_ok!(
     get("/").on(&handler),
-    "<h1>hello world</h1>",
+    "<html>\n  <head>\n    <script src=\"/js.js\"></script>\n  </head>\n  <body>\n    <h1>hello world</h1>\n  </body>\n</html>",
     "content-type" => "text/html"
 );
 assert_not_handled!(get("/file_that_does_not_exist.txt").on(&handler));
 assert_ok!(get("/index.html").on(&handler));
-assert_ok!(get("/subdir/index.html").on(&handler), "subdir index.html");
-assert_ok!(get("/subdir").on(&handler), "subdir index.html");
+assert_ok!(
+    get("/subdir/index.html").on(&handler),
+    "subdir index.html 🎈",
+    "content-type" => "text/html; charset=utf-8"
+);
+assert_ok!(get("/subdir").on(&handler), "subdir index.html 🎈");
 assert_not_handled!(get("/subdir_with_no_index").on(&handler));
 assert_ok!(
     get("/subdir_with_no_index/plaintext.txt").on(&handler),
@@ -111,9 +115,25 @@ impl StaticCompiledHandler {
     }
 
     fn serve_file(&self, mut conn: Conn, file: File<'static>) -> Conn {
-        if let Some(mime) = mime_db::lookup(file.path().to_string_lossy().as_ref()) {
-            conn.headers_mut().try_insert(ContentType, mime);
-        }
+        let mime = mime_guess::from_path(file.path()).first_or_text_plain();
+
+        let is_ascii = file.contents().is_ascii();
+        let is_text = match (mime.type_(), mime.subtype()) {
+            (mime::APPLICATION, mime::JAVASCRIPT) => true,
+            (mime::TEXT, _) => true,
+            (_, mime::HTML) => true,
+            _ => false,
+        };
+
+        conn.headers_mut().try_insert(
+            ContentType,
+            if is_text && !is_ascii {
+                format!("{}; charset=utf-8", mime)
+            } else {
+                mime.to_string()
+            },
+        );
+
         conn.ok(file.contents())
     }
 
